#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include "RFM73.h"

void ioinit(void);

void power_on_delay(void);

void Send_Packet(uint8_t type,uint8_t* pbuf,uint8_t len);
void Receive_Packet(void);

extern void RFM73_Initialize(void);
extern void SwitchToTxMode(void);
extern void SwitchToRxMode(void);

uint8_t tx_buf[MAX_PACKET_LEN]; 
uint8_t rx_buf[MAX_PACKET_LEN];

void power_on_delay(void)
{
	_delay_ms(1000);
}

void Send_Packet(uint8_t type,uint8_t* pbuf,uint8_t len)
{
	uint8_t fifo_sta;
	
	SwitchToTxMode();

	fifo_sta=SPI_Read_Reg(FIFO_STATUS);
	if((fifo_sta&FIFO_STATUS_TX_FULL)==0)
	{
		SPI_Write_Buf(type, pbuf, len);
		
		pbuf[len-1] = '\0';	

		PORTB |= _BV(0);
		_delay_ms(100);
		PORTB &= ~_BV(0);
	}	  	 	
}

void Receive_Packet(void)
{
	uint8_t len,rx_count,j,sta,fifo_sta,chksum;
	uint8_t rx_buf[MAX_PACKET_LEN];

	rx_count = 0;	
	sta=SPI_Read_Reg(STATUS);

	if((STATUS_RX_DR&sta) == 0x40)
	{
		do
		{
			len=SPI_Read_Reg(R_RX_PL_WID_CMD);

			if(len<=MAX_PACKET_LEN)
			{
				SPI_Read_Buf(RD_RX_PLOAD,rx_buf,len);
				rx_count += len;
			}
			else
			{
				SPI_Write_Reg(FLUSH_RX,0);
			}

			fifo_sta=SPI_Read_Reg(FIFO_STATUS);
			
		} while((fifo_sta&FIFO_STATUS_RX_EMPTY)==0);
		
		rx_buf[rx_count] = '\0';
		chksum = 0;
		for(j=0;j<(rx_count-1);j++)
		{
			chksum += rx_buf[j]; 
		}

		if(chksum==rx_buf[rx_count-1])
		{
			PORTB |= _BV(0);
			_delay_ms(100);
			PORTB &= ~_BV(0);
			SwitchToRxMode();
			rx_buf[len-1] = '\0';
		}	
	}
	SPI_Write_Reg(WRITE_REG|STATUS,sta);	
}

void ioinit(void)
{
    DDRB = 0b11101111;
	DDRD |= _BV(4);
	DDRD |= _BV(5);
	SPCR |= 0b01010011;
	PORTB &= ~_BV(0);
}

void send()
{
	cli();
	PORTD |= _BV(4);
	_delay_ms(100);
	tx_buf[0] = 'a';
	Send_Packet(W_TX_PAYLOAD_NOACK_CMD,tx_buf,(uint8_t)strlen(tx_buf));
	tx_buf[0] = '\0';
	sei();
	SwitchToRxMode();
}

int main()
{
	ioinit();
    PORTB |= _BV(0);
	_delay_ms(250);
    PORTB &= ~_BV(0);
	
    PORTD |= _BV(4);
	_delay_ms(250);
    PORTD &= ~_BV(4);
	
	tx_buf[0] = '\0';
	sei();
	power_on_delay();

	RFM73_Initialize();
	
	while (1)
	{
		Receive_Packet();
	}
	
	return 0;
}